/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import static org.eclipse.jface.resource.FontDescriptor.createFrom;

import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.base.model.element.ConstraintConfiguration;
import org.fortiss.tooling.kernel.extension.IConstraint;

// TODO(SB,3): Add class comment, use proper author name tag (fortiss login)
/**
 * 
 * @author ss
 */
public class ConstraintBasedProcessEditorHelper {

	/** Color Array for at least seven groups. */
	static int[] colorArray = {SWT.COLOR_DARK_GREEN, SWT.COLOR_DARK_BLUE, SWT.COLOR_DARK_YELLOW,
			SWT.COLOR_BLACK, SWT.COLOR_DARK_RED, SWT.COLOR_GREEN, SWT.COLOR_CYAN};

	/** Counter for the colorArray */
	public static int colorCtr = 0;

	/** Reference to a constraint configuration. */
	public static class WithContextualConfiguration {

		/** Contextual constraint configuration. */
		ConstraintConfiguration config;

		/** Constructor. */
		public WithContextualConfiguration(ConstraintConfiguration config) {
			this.config = config;
		}
	}

	/** Constraint accompanied with a configuration providing the context. */
	public static class ConstraintWithContextualConfiguration extends WithContextualConfiguration {

		/** Constraint. */
		Class<? extends IConstraint> cstr;

		/** Constructor. */
		public ConstraintWithContextualConfiguration(Class<? extends IConstraint> cstr,
				ConstraintConfiguration config) {
			super(config);
			this.cstr = cstr;
		}
	}

	/**
	 * Configuration accompanied with another configuration providing the context.
	 * Intent is that the contained configuration is a dependency of the context one.
	 */
	public static class ConfigurationRefWithContextualConfiguration extends
			WithContextualConfiguration {

		/** Constraint. */
		// TODO: Check if the variable should really be package accessible
		/* package */ConstraintConfiguration target;

		/** Constructor. */
		public ConfigurationRefWithContextualConfiguration(ConstraintConfiguration configRef,
				ConstraintConfiguration config) {
			super(config);
			this.target = configRef;
		}

		/** {@inheritDoc} */
		@Override
		public boolean equals(Object obj) {
			if(obj instanceof ConfigurationRefWithContextualConfiguration) {
				// TODO (SB, 19)
				return this.target.getName().equalsIgnoreCase(
						((ConfigurationRefWithContextualConfiguration)obj).target.getName()) &&
						this.config.getName()
								.equalsIgnoreCase(
										((ConfigurationRefWithContextualConfiguration)obj).config
												.getName());
			}
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public int hashCode() {
			return target.hashCode();
		}
	}

	/** Class to manipulate and store parent of elements. */
	public static class TreeStructureNode {

		// TODO: Check if the variables should really be package accessible
		/** Parent object variable. */
		/* package */Object parent;

		/** Data object variable. */
		/* package */Object data;

		/** Constructor. */
		public TreeStructureNode(Object parent, Object children) {
			this.parent = parent;
			this.data = children;
		}

		/** {@inheritDoc} */
		@Override
		public boolean equals(Object obj) {
			if(obj instanceof TreeStructureNode) {
				// TODO (SB, 19)
				return this.data.toString().equalsIgnoreCase(
						((TreeStructureNode)obj).data.toString()) &&
						this.parent.toString().equalsIgnoreCase(
								((TreeStructureNode)obj).parent.toString());
			}
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public int hashCode() {
			return parent.hashCode();
		}
	}

	/** Styler class to create colored bold labels for constraint group name. */
	public static class NormalFontStyler extends Styler {

		/** Counter to keep track of currently used color in color array. */
		// TODO: Check if the variable should really be package accessible
		/* package */int colorIndex = 0;

		/** Constructor. */
		public NormalFontStyler() {
			this.colorIndex = colorCtr;
		}

		/** {@inheritDoc} */
		@Override
		public void applyStyles(final TextStyle textStyle) {

			FontDescriptor boldDescriptor = createFrom(new FontData()).setStyle(SWT.BOLD);
			Font boldFont = boldDescriptor.createFont(Display.getCurrent());
			textStyle.font = boldFont;
			textStyle.foreground = Display.getCurrent().getSystemColor(colorArray[colorIndex]);
		}
	}

	// TODO (SB,3)
	/** */
	public static class RedFontStyler extends Styler {
		/** {@inheritDoc} */
		@Override
		public void applyStyles(final TextStyle textStyle) {
			FontDescriptor boldDescriptor = createFrom(new FontData()).setStyle(SWT.BOLD);
			Font boldFont = boldDescriptor.createFont(Display.getCurrent());
			textStyle.font = boldFont;
			textStyle.foreground = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_RED);
		}
	}

	/** Styler class to create green color labels. */
	public static class GreenFontStyler extends Styler {
		/** {@inheritDoc} */
		@Override
		public void applyStyles(final TextStyle textStyle) {
			FontDescriptor boldDescriptor = createFrom(new FontData()).setStyle(SWT.BOLD);
			Font boldFont = boldDescriptor.createFont(Display.getCurrent());
			textStyle.font = boldFont;
			textStyle.foreground = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GREEN);
		}
	}

	// TODO(SB,3)
	/**
	 * @param groupSize
	 *            the size of the groups present
	 */
	public static void adjustColorIndex(int groupSize) {
		// TODO(SB,11): Can you use modulo to write this more elegantly?
		if(ConstraintBasedProcessEditorHelper.colorCtr != ConstraintBasedProcessEditorHelper.colorCtr - 1) {
			if(groupSize == ConstraintBasedProcessEditorHelper.colorCtr + 1) {
				ConstraintBasedProcessEditorHelper.colorCtr = 0;
			} else {
				ConstraintBasedProcessEditorHelper.colorCtr++;
			}
		}
	}
}
